package com.greenbird.test.util;

import org.springframework.util.ReflectionUtils;

import java.lang.reflect.Method;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.CoreMatchers.nullValue;
import static org.hamcrest.core.IsNot.not;
import static org.junit.Assert.assertThat;

public class EqualsTester {
    @SuppressWarnings("ObjectEqualsNull")
    public static <T> void testEquals(T objectToCompare, T unequalObject) {
        assertThat(objectToCompare, not(nullValue()));

        Method equalsMethod = ReflectionUtils.findMethod(objectToCompare.getClass(), "equals", Object.class);
        Class declaringClass = equalsMethod.getDeclaringClass();
        Class objectClass = Object.class;
        assertThat(objectToCompare + " must not use default equals implementation.", declaringClass, not(objectClass));

        assertThat("Equals must test on identify.", objectToCompare.equals(objectToCompare), is(true));

        assertThat("Equals must test on null argument.", objectToCompare.equals(null), is(false));

        assertThat("Equals must test on class type.", objectToCompare.equals("test"), is(false));

        assertThat("Equals must return false on unequal argument.", objectToCompare.equals(unequalObject), is(false));
    }
}
