package com.greenbird.test.web;

import com.greenbird.test.GreenbirdTestException;
import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;
import com.sun.net.httpserver.HttpServer;
import org.apache.commons.io.IOUtils;
import org.apache.mina.util.AvailablePortFinder;

import java.io.IOException;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class HttpTestServer {
    private HttpServer httpServer;
    private int port;
    private List<HttpResponseStub> responses = new ArrayList<HttpResponseStub>();

    public HttpTestServer() {
        this(AvailablePortFinder.getNextAvailable(1025));
    }

    public HttpTestServer(int port) {
        this.port = port;
        try {
            httpServer = HttpServer.create(new InetSocketAddress("localhost", port), 0);
        } catch (IOException e) {
            throw new GreenbirdTestException(e);
        }

        httpServer.createContext("/", new HttpHandler() {
            @Override
            public void handle(HttpExchange httpExchange) throws IOException {
                if (!responses.isEmpty()) {
                    sendStubResponse(httpExchange);
                } else {
                    sendNotFoundResponse(httpExchange);
                }
            }

            private void sendStubResponse(HttpExchange httpExchange) throws IOException {
                HttpResponseStub responseStub = responses.remove(0);
                httpExchange.getResponseHeaders().set("Content-Type", responseStub.getContentType());
                httpExchange.sendResponseHeaders(responseStub.getStatusCode(), 0);
                OutputStream responseBodyStream = httpExchange.getResponseBody();
                IOUtils.write(responseStub.getContent(), responseBodyStream);
                responseBodyStream.close();
            }

            private void sendNotFoundResponse(HttpExchange httpExchange) throws IOException {
                httpExchange.sendResponseHeaders(404, 0);
                OutputStream responseBodyStream = httpExchange.getResponseBody();
                IOUtils.write("Test error: No more response stubs configured.", responseBodyStream);
                responseBodyStream.close();
            }
        });
        httpServer.setExecutor(null);
        httpServer.start();
    }

    public void stop() {
        httpServer.stop(0);
    }

    public int getPort() {
        return port;
    }

    public void addResponses(HttpResponseStub... responseStubs) {
        responses.addAll(Arrays.asList(responseStubs));
    }
}
